package ch.zhaw.prog2.functional.streaming.finance;

import ch.zhaw.prog2.functional.streaming.Company;
import ch.zhaw.prog2.functional.streaming.humanresource.Employee;

import java.util.Currency;
import java.util.List;
import java.util.stream.Stream;

/**
 * This Class creates a Payroll (Lohabrechnung) for a whole Company
 * and supplies some Utility Methods for a a Payroll.
 * ✅  This class should be worked on by students.
 */
public class PayrollCreator {
    private final Company company;

    /**
     * Opens a Payroll for a company.
     * @param company
     */
    public PayrollCreator(Company company) {
        this.company = company;
    }

    /*
     * Aufgabe d) - Test dazu exisitert in PayrollCreatorTest
     */
    public Payroll getPayrollForAll() {
        Payroll payroll = new Payroll();
        payroll.addPayments(company.getPayments(Employee::isWorkingForCompany));
        return payroll;
    }

    /*
     * Aufgabe e) - Test dazu existiert in PayrollCreatorTest
     */
    public static int payrollValueCHF(Payroll payroll) {
       return payroll.stream().map(payment -> CurrencyChange.getInNewCurrency(payment.getCurrencyAmount(), Currency.getInstance("CHF")).getAmount()).reduce(Integer::sum).orElse(0);
    }

    /**
     * Aufgabe f) - schreiben Sie einen eigenen Test in PayrollCreatorTestStudent
     * @return a List of total amounts in this currency for each currency in the payroll
     */
    public static List<CurrencyAmount> payrollAmountByCurrency(Payroll payroll) {
        List<CurrencyAmount> currencyAmounts = payroll.stream().map(Payment::getCurrencyAmount).toList();
        return currencyAmounts.stream()
            .map(CurrencyAmount::getCurrency)
            .distinct()
            .map(currency -> new CurrencyAmount(
                    currencyAmounts.stream()
                        .filter(currencyAmount -> currencyAmount.getCurrency().equals(currency))
                        .map(CurrencyAmount::getAmount)
                        .reduce(Integer::sum)
                        .orElse(0), currency))
            .toList();
    }


}
